#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>

#include "ScriptVisualizerApp.h"

ScriptVisualizerApp::ScriptVisualizerApp	(void)
 : BApplication	("application/x-vnd.Kirilla-ScriptVisualizer")	
{
	argv_received	=	false;
	refs_received	=	false;
}

ScriptVisualizerApp::~ScriptVisualizerApp	(void)	
{
	// void
}

void 
ScriptVisualizerApp::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	BMessage *	msg = new BMessage(B_REFS_RECEIVED);
	int32	result;

	if(argc <= 1)	// no arguments
		return;

//	get current path

	char *	buffer = new char[4096];
	getcwd(buffer, 4096);
	BString	currentdir = buffer;
	if (currentdir.FindLast('/') != currentdir.Length()-1)
		currentdir.Append("/");
	delete buffer;

//	convert arguments to entry_refs

	for(int32 i = 1; i < argc; i++)
	{
		BString path_str = argv[i];
		result =  path_str.FindFirst("/");
		
		if (result == B_ERROR || result != 0)
		{
			path_str.Prepend(currentdir);
		}
			
		BEntry		entry(path_str.String());
		entry_ref	ref;
		entry.GetRef(& ref);
		
		msg->AddRef("refs", & ref);

	}
	
//	Send to RefsReceived
	RefsReceived(msg);
	delete msg;
}

void 
ScriptVisualizerApp::RefsReceived	(BMessage * a_message)
{
	refs_received	=	true;
	
	ProcessRefs(a_message);
}

void
ScriptVisualizerApp::ReadyToRun	(void)
{
	if (!argv_received && !refs_received)
	{
		(new BAlert("ScriptVisualizer", "You can use ScriptVisualizer in 3 ways:\n"
		"1. Drop scripts on the icon\n"
		"2. Select Open With... in Tracker's right-click menu\n"
		"3. Set your individual script files to open in ScriptVisualizer "
		"(using the FileType addon)", "Alright"))->Go();
	
	}

	PostMessage(B_QUIT_REQUESTED);
}

void 
ScriptVisualizerApp::ProcessRefs	(BMessage * a_message)
{
	entry_ref	ref;
	int32		ref_count	=	0;

	while(a_message->FindRef("refs", ref_count, &ref) == B_OK)
	{
		RunScript(& ref);
		ref_count++;	
	}
}

void 
ScriptVisualizerApp::RunScript(const entry_ref * a_ref)	
{
	BPath	path	(a_ref);
	BString	string	(path.Path());
	NameSafe(& string);
	string.Prepend("/boot/beos/apps/Terminal -t \"Script Visualizer\" /bin/sh ");
	string.Append(" wait &");
	system(string.String());
}

void
ScriptVisualizerApp::NameSafe	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == '\"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$')  { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`')  { a_string-> Insert ("\\",position); position++; }
		position++;
	}

	a_string->Prepend("\"");
	a_string->Append("\"");
}
